
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_poly.h>
#include <gsl/gsl_complex_math.h>


static double tic(void) { 
    return (clock()/ (double) CLOCKS_PER_SEC); 
}

static double toc(double t) { 
    double s = tic(); 
    if(s<t) return(0.0);
    return(s-t); 
}

double my_pow_int(double x, int n) {
    double z = x;
    int    j;
    for(j=2; j<=n; ++j) 
        z *= x;
    return(z);
}

int main(int argc, char **argv)      {
    double      x, y, u, tini, tfin;
    int         i, ntimes;
    gsl_complex z, z5, zdiff, zunit;

    printf("\n--------- Calculo sin usar GSL: ------------\n");
    x = 5.0e200;
    y = 8.0e200;
    u = sqrt(x*x + y*y);

    printf("u = sqrt( (%4.1e)^2 + (%4.1e)^2) = %e\n", x, y, u);

    if(gsl_isinf(u))
       printf("Desbordamiento al calcular u\n"); 

    printf("\n--------- Calculo usando GSL: ------------\n");
    u = gsl_hypot(x, y);
    printf("u = sqrt( (%4.1e)^2 + (%4.1e)^2) = %e\n", x, y, u);

    if(gsl_isinf(u))
       printf("Desbordamiento al calcular u\n\n"); 
    if(gsl_finite(u))
       printf("gsl_hypot(x,y) hace el calculo sqrt(x^2+y^2) de forma robusta\n\n"); 


    printf("\n--------- Potencia entera de un numero ------------\n");
    printf("gsl_pow_int(x,n) lo hace de forma eficiente. Por ejemplo,\n");
    printf("x^8 lo calcula como ((x^2)^2)^2 i.e. solo usa 3 productos.\n");
    x = 0.5;
    ntimes = 10000000;
    tini = tic();    
    for(i=0; i<ntimes; ++i) 
        u = my_pow_int(x, 8);
    tfin = toc(tini);
    printf("Tiempo para calcular (%4.2f)^8 = %8.6f %d veces: %6.4f seg. sin GSL\n\n", x, u, ntimes, tfin);

    tini = tic();    
    for(i=0; i<ntimes; ++i) 
        u = gsl_pow_int(x, 8);
    tfin = toc(tini);
    printf("Tiempo para calcular (%4.2f)^8 = %8.6f %d veces: %6.4f seg. con GSL\n\n", x, u, ntimes, tfin);



    printf("\n--------- Calculo de raices de un polinomio ------------\n");
    printf("Raices del polinomio de grado 5:  -1 + x^5\n");
    printf("Tomamos en cuenta las raices complejas\n");
    int    ncoef = 6;
    double a[6] = {-1, 0, 0, 0, 0, 1 };
    // Reservamos espacio para la partes real e imaginaria de las 5 raices
    double roots[10];   
    gsl_poly_complex_workspace *w = gsl_poly_complex_workspace_alloc(ncoef);

    gsl_poly_complex_solve(a, ncoef, w, roots);
    gsl_poly_complex_workspace_free(w);
    zunit = gsl_complex_rect(1, 0);  // = 1 + 0i  
    for(i=0; i<5; i++) {
        printf("z%d = %+.10f %+.10f * i  ", i, roots[2*i], roots[2*i+1]);
        // Creamos el numero complejo usando la parte real e imaginaria
        z    = gsl_complex_rect(roots[2*i], roots[2*i+1]);
        // Elevamos z a la quinta potencia. Deberia ser 1
        z5    = gsl_complex_pow_real(z, 5);
        zdiff = gsl_complex_sub(z5, zunit);
        printf(" ==> |z%d^5 - 1| = %e\n", i, gsl_complex_abs(zdiff));
    }

    return(0);
}
// Para compilar
// gcc -o pruebaGsl pruebaGsl.c  -lgsl -lgslcblas -lm

// Para ejecutar
// ./pruebaGsl

